﻿using IRaCIS.Core.Domain.Share;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Filters;
using Microsoft.Extensions.Logging;

namespace IRaCIS.Core.Application.Service.BusinessFilter;


#region 控制器流程
/// <summary>
///  统一返回前端数据包装，之前在控制器包装，现在修改为动态Api  在ResultFilter这里包装，减少重复冗余代码
/// by zhouhang 2021.09.12 周末
/// </summary>
public class UnifiedApiResultFilter : Attribute, IAsyncResultFilter
{
    private readonly ILogger _logger;

    public UnifiedApiResultFilter(ILogger<UnifiedApiResultFilter> logger)
    {
        _logger = logger;
    }

    /// <summary>
    /// 异步版本
    /// </summary>
    /// <param name="context"></param>
    /// <param name="next"></param>
    /// <returns></returns>
    public async Task OnResultExecutionAsync(ResultExecutingContext context, ResultExecutionDelegate next)
    {

        if (context.Result is ObjectResult objectResult)
        {
            var statusCode = objectResult.StatusCode ?? context.HttpContext.Response.StatusCode;

            //是200 并且没有包装  那么包装结果
            if (statusCode == 200 && !(objectResult.Value is IResponseOutput))
            {

                var type = objectResult.Value?.GetType();


                if (type != null && type.IsGenericType && (type.GetGenericTypeDefinition() == typeof(ValueTuple<,>) || type.GetGenericTypeDefinition() == typeof(Tuple<,>)))
                {

                    //报错
                    //var tuple = (object, object))objectResult.Value;
                    //var (val1, val2) = ((dynamic, dynamic))objectResult.Value;
                    //var apiResponse = ResponseOutput.Ok(val1, val2);

                    //OK
                    var tuple = (dynamic)objectResult.Value;
                    var apiResponse = ResponseOutput.Ok(tuple!.Item1, tuple!.Item2);


                    objectResult.Value = apiResponse;
                    objectResult.DeclaredType = apiResponse.GetType();
                }
                else
                {
                    var apiResponse = ResponseOutput.Ok(objectResult.Value);

                    objectResult.Value = apiResponse;
                    objectResult.DeclaredType = apiResponse.GetType();
                }


            }
            //如果是200  是IResponseOutput  记录下日志
            else if (statusCode == 200 && (objectResult.Value is IResponseOutput))
            {
                var result = objectResult.Value as IResponseOutput;

                if (!string.IsNullOrWhiteSpace(result.LocalizedInfo))
                {
                    //统一在这里记录国际化的日志信息
                    _logger.LogWarning($"{result.LocalizedInfo}");
                }

            }

            else if (statusCode != 200 && !(objectResult.Value is IResponseOutput))
            {
                //---程序错误，请联系开发人员。
                var apiResponse = ResponseOutput.NotOk(StaticData.International("UnifiedAPI_ProgramError"));

                objectResult.Value = apiResponse;
                objectResult.DeclaredType = apiResponse.GetType();
            }

        }

        await next.Invoke();

    }

    public static bool IsTupleType(Type type, bool checkBaseTypes = false)
    {
        if (type == null)
            throw new ArgumentNullException(nameof(type));

        if (type == typeof(Tuple))
            return true;

        while (type != null)
        {
            if (type.IsGenericType)
            {
                var genType = type.GetGenericTypeDefinition();
                if (genType == typeof(Tuple<>)
                    || genType == typeof(Tuple<,>)
                    || genType == typeof(Tuple<,>))
                    return true;
            }

            if (!checkBaseTypes)
                break;

            type = type.BaseType;
        }

        return false;
    }
}
#endregion


#region minimalapi 流程


public class UnifiedApiResultEndpointFilter : IEndpointFilter
{
    private readonly ILogger _logger;

    public UnifiedApiResultEndpointFilter(ILogger<UnifiedApiResultFilter> logger)
    {
        _logger = logger;
    }

    public async ValueTask<object?> InvokeAsync(EndpointFilterInvocationContext context, EndpointFilterDelegate next)
    {
        // 调用下一个过滤器或端点处理程序
        var result = await next(context);

        // 检查返回值类型
        if (result is null)
        {
            return ResponseOutput.NotOk("No data found."); // 处理 null 返回
        }

        // 如果返回的是元组
        if (result is ValueTuple<object, object> tuple)
        {
            return ResponseOutput.Ok(tuple.Item1, tuple.Item2);
        }

        if (result is IResponseOutput)
        {
            return result;
        }

        // 对于其他情况，直接返回结果
        return ResponseOutput.Ok(result);
    }

    private IResponseOutput WrapResponse(object? value)
    {
        // 处理元组的情况
        if (value is ValueTuple<object, object> tuple)
        {
            return ResponseOutput.Ok(tuple.Item1, tuple.Item2);
        }

        // 包装单个对象
        return ResponseOutput.Ok(value);
    }
}

#endregion

